@@ -3,6 +3,7 @@ import warnings
 
 import numpy as np
 import pandas as pd
+import shapely.errors
 from pandas import DataFrame, Series
 from pandas.core.accessor import CachedAccessor
 
@@ -921,13 +922,13 @@ individually so that features may have different properties
         if len(properties_cols) > 0:
             # convert to object to get python scalars.
             properties_cols = self[properties_cols]
-            properties = properties_cols.astype(object).values
+            properties = properties_cols.astype(object)
             na_mask = pd.isna(properties_cols).values
 
             if na == "null":
                 properties[na_mask] = None
 
-            for i, row in enumerate(properties):
+            for i, row in enumerate(properties.values):
                 geom = geometries[i]
 
                 if na == "drop":
@@ -938,7 +939,7 @@ individually so that features may have different properties
                         if not na
                     }
                 else:
-                    properties_items = {k: v for k, v in zip(properties_cols, row)}
+                    properties_items = dict(zip(properties_cols, row))
 
                 if drop_id:
                     feature = {}
@@ -1431,8 +1432,6 @@ individually so that features may have different properties
 
         .. versionadded:: 0.9
 
-        .. note:: Requires pyproj 3+
-
         Parameters
         ----------
         datum_name : str, optional
@@ -1529,12 +1528,18 @@ individually so that features may have different properties
             if pd.api.types.is_scalar(value) or isinstance(value, BaseGeometry):
                 value = [value] * self.shape[0]
             try:
-                crs = getattr(self, "crs", None)
+                if self._geometry_column_name is not None:
+                    crs = getattr(self, "crs", None)
+                else:  # don't use getattr, because a col "crs" might exist
+                    crs = None
                 value = _ensure_geometry(value, crs=crs)
                 if key == "geometry":
                     self._persist_old_default_geometry_colname()
             except TypeError:
-                warnings.warn("Geometry column does not contain geometry.")
+                warnings.warn(
+                    "Geometry column does not contain geometry.",
+                    stacklevel=2,
+                )
         super().__setitem__(key, value)
 
     #
@@ -1610,7 +1615,7 @@ individually so that features may have different properties
                     # not enough info about func to preserve CRS
                     result = _ensure_geometry(result)
 
-                except TypeError:
+                except (TypeError, shapely.errors.GeometryTypeError):
                     pass
 
         return result
@@ -1693,9 +1698,12 @@ individually so that features may have different properties
 
         Parameters
         ----------
-        by : string, default None
-            Column whose values define groups to be dissolved. If None,
-            whole GeoDataFrame is considered a single group.
+        by : str or list-like, default None
+            Column(s) whose values define the groups to be dissolved. If None,
+            the entire GeoDataFrame is considered as a single group. If a list-like
+            object is provided, the values in the list are treated as categorical
+            labels, and polygons will be combined based on the equality of
+            these categorical labels.
         aggfunc : function or string, default "first"
             Aggregation function for manipulation of data associated
             with each group. Passed to pandas `groupby.agg` method.
@@ -1770,9 +1778,13 @@ individually so that features may have different properties
         if by is None and level is None:
             by = np.zeros(len(self), dtype="int64")
 
-        groupby_kwargs = dict(
-            by=by, level=level, sort=sort, observed=observed, dropna=dropna
-        )
+        groupby_kwargs = {
+            "by": by,
+            "level": level,
+            "sort": sort,
+            "observed": observed,
+            "dropna": dropna,
+        }
 
         # Process non-spatial component
         data = self.drop(labels=self.geometry.name, axis=1)
@@ -2098,7 +2110,6 @@ individually so that features may have different properties
 
     @doc(_explore)
     def explore(self, *args, **kwargs):
-        """Interactive map based on folium/leaflet.js"""
         return _explore(self, *args, **kwargs)
 
     def sjoin(self, df, *args, **kwargs):
@@ -2175,7 +2186,7 @@ individually so that features may have different properties
         GeoDataFrame.sjoin_nearest : nearest neighbor join
         sjoin : equivalent top-level function
         """
-        return geopandas.sjoin(left_df=self, right_df=df, *args, **kwargs)
+        return geopandas.sjoin(left_df=self, right_df=df, *args, **kwargs)  # noqa: B026
 
     def sjoin_nearest(
         self,
@@ -2185,6 +2196,7 @@ individually so that features may have different properties
         lsuffix="left",
         rsuffix="right",
         distance_col=None,
+        exclusive=False,
     ):
         """
         Spatial join of two GeoDataFrames based on the distance between their
@@ -2222,6 +2234,10 @@ individually so that features may have different properties
         distance_col : string, default None
             If set, save the distances computed between matching geometries under a
             column of this name in the joined GeoDataFrame.
+        exclusive : bool, optional, default False
+            If True, the nearest geometries that are equal to the input geometry
+            will not be returned, default False.
+            Requires Shapely >= 2.0
 
         Examples
         --------
@@ -2304,6 +2320,7 @@ chicago_w_groceries[chicago_w_groceries["community"] == "UPTOWN"]
             lsuffix=lsuffix,
             rsuffix=rsuffix,
             distance_col=distance_col,
+            exclusive=exclusive,
         )
 
     def clip(self, mask, keep_geom_type=False):
@@ -2409,8 +2426,8 @@ chicago_w_groceries[chicago_w_groceries["community"] == "UPTOWN"]
         1       2.0       1.0  POLYGON ((2.00000 2.00000, 2.00000 3.00000, 3....
         2       2.0       2.0  POLYGON ((4.00000 4.00000, 4.00000 3.00000, 3....
         3       1.0       NaN  POLYGON ((2.00000 0.00000, 0.00000 0.00000, 0....
-        4       2.0       NaN  MULTIPOLYGON (((3.00000 3.00000, 4.00000 3.000...
-        5       NaN       1.0  MULTIPOLYGON (((2.00000 2.00000, 3.00000 2.000...
+        4       2.0       NaN  MULTIPOLYGON (((3.00000 4.00000, 3.00000 3.000...
+        5       NaN       1.0  MULTIPOLYGON (((2.00000 3.00000, 2.00000 2.000...
         6       NaN       2.0  POLYGON ((3.00000 5.00000, 5.00000 5.00000, 5....
 
         >>> df1.overlay(df2, how='intersection')
@@ -2422,14 +2439,14 @@ chicago_w_groceries[chicago_w_groceries["community"] == "UPTOWN"]
         >>> df1.overlay(df2, how='symmetric_difference')
         df1_data  df2_data                                           geometry
         0       1.0       NaN  POLYGON ((2.00000 0.00000, 0.00000 0.00000, 0....
-        1       2.0       NaN  MULTIPOLYGON (((3.00000 3.00000, 4.00000 3.000...
-        2       NaN       1.0  MULTIPOLYGON (((2.00000 2.00000, 3.00000 2.000...
+        1       2.0       NaN  MULTIPOLYGON (((3.00000 4.00000, 3.00000 3.000...
+        2       NaN       1.0  MULTIPOLYGON (((2.00000 3.00000, 2.00000 2.000...
         3       NaN       2.0  POLYGON ((3.00000 5.00000, 5.00000 5.00000, 5....
 
         >>> df1.overlay(df2, how='difference')
                                                 geometry  df1_data
         0  POLYGON ((2.00000 0.00000, 0.00000 0.00000, 0....         1
-        1  MULTIPOLYGON (((3.00000 3.00000, 4.00000 3.000...         2
+        1  MULTIPOLYGON (((3.00000 4.00000, 3.00000 3.000...         2
 
         >>> df1.overlay(df2, how='identity')
         df1_data  df2_data                                           geometry
@@ -2437,7 +2454,7 @@ chicago_w_groceries[chicago_w_groceries["community"] == "UPTOWN"]
         1       2.0       1.0  POLYGON ((2.00000 2.00000, 2.00000 3.00000, 3....
         2       2.0       2.0  POLYGON ((4.00000 4.00000, 4.00000 3.00000, 3....
         3       1.0       NaN  POLYGON ((2.00000 0.00000, 0.00000 0.00000, 0....
-        4       2.0       NaN  MULTIPOLYGON (((3.00000 3.00000, 4.00000 3.000...
+        4       2.0       NaN  MULTIPOLYGON (((3.00000 4.00000, 3.00000 3.000...
 
         See also
         --------
